@@ -433,7 +433,7 @@ public:
 
     hyFloat  FisherExact             (hyFloat, hyFloat, hyFloat);
 
-    virtual     bool        HasChanged  (bool = false);
+    virtual     bool        HasChanged  (bool = false, _AVLListX * cache = nil);
     // have any variables which are referenced by the elements changed?
 
     virtual     unsigned long
@@ -715,6 +715,7 @@ private:
 void _hy_matrix_multiply_4x4 (double * C, double *A, double *B, int stride, bool add);
 void _hy_matrix_multiply_NxN_blocked4 (double * C, double *A, double *B, int D);
 void _hy_matrix_transpose_blocked (double * __restrict C, double * __restrict A, int nrow, int ncol);
+
 /*__________________________________________________________________________________________________________________________________________ */
 
 extern  _Matrix *GlobalFrequenciesMatrix;
